@@ -33,22 +33,8 @@
 # undef info
 #endif
 
-
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
-extern __inline__ int atomic_inc_and_test_greater_zero (volatile atomic_t *v)
-{
-        unsigned char c;
-
-        __asm__ __volatile__(
-                LOCK "incl %0; setg %1"
-                :"=m" (v->counter), "=qm" (c)
-                :"m" (v->counter) : "memory");
-        return c; /* can be only 0 or 1 */
-}
-#endif
-
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,4)
 #define VSNPRINTF(s,n,f,a)	vsnprintf (s, n, f, a)
 #define SNPRINTF(s,n,f,l...)	snprintf (s, n, f, ##l)
@@ -57,8 +43,6 @@ extern __inline__ int atomic_inc_and_test_greater_zero (volatile atomic_t *v)
 #define SNPRINTF(s,n,f,l...)	sprintf (s, f, ##l)
 #endif
 
-extern unsigned long atomic_xchg (volatile atomic_t * v, unsigned value);
-
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 #ifndef NDEBUG
@@ -95,6 +79,55 @@ extern void *   hcalloc (unsigned size);
 
 extern void     hfree (void * mem);
 
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+
+#if !defined (NDEBUG) && defined (LOG_TIMER)
+
+#include <linux/time.h>
+
+typedef struct {
+
+	const char *	name;
+	struct timeval	t;
+	struct timeval	d;
+} dbg_timer;
+
+#define	PRINT_TIMER(x)		log ( \
+					"Timer '%s': %ld s, %ld s\n", \
+					(x)->name, \
+					(x)->t.tv_sec, (x)->t.tv_usec \
+				)
+#define	PRINT_TIME_MSG(s,x)	log ( \
+					"%s: %ld s, %ld s\n", \
+					s, (x)->t.tv_sec, (x)->t.tv_usec \
+				)
+
+#define	WATCH_DECL(id)		dbg_timer	id##_timer
+#define	WATCH_START(id)		start_watch (&id##_timer)
+#define	WATCH_STOP(id)		stop_watch (&id##_timer); \
+				PRINT_TIME_MSG(#id,&id##_timer)
+	
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+extern int	timers_start (void);
+extern void	timers_stop (void);
+
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+extern void	setup_timer (dbg_timer * t, long dsec, long dusec);
+extern int	check_timer (dbg_timer * t);
+extern void	touch_timer (dbg_timer * t);
+
+extern void	start_watch (dbg_timer * w);
+extern void	stop_watch (dbg_timer * w);
+#else
+#define	PRINT_TIMER(t)
+#define	WATCH_DECL(name)
+#define	WATCH_START(name)
+#define	WATCH_STOP(name)
+#endif
+
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 #ifdef NEED_MEM_STR_PROTOS
